unit hiRegExp;

interface

uses Kol,Share,Debug,KOLComObj;

type
  THIRegExp = class(TDebug)
   private
    objRegExp:OleVariant;
    objMatches:OleVariant;
    objMatch:OleVariant;
    fPattern:string;
    fValue:string;
    fPosition:integer;
    fLength:integer;
    fpos,fmax:integer;
    fList:PStrList;
    fStop:boolean;

    procedure SetProp(var obj: OleVariant);

   public
    _prop_Pattern:string;
    _prop_Global:integer;
    _prop_IgnoreCase:integer;
    _prop_Multiline:integer;
    _prop_PatternList:string;


    _data_strSource:THI_Event;
    _data_strReplace:THI_Event;
    _data_Pattern:THI_Event;
    _data_Global:THI_Event;
    _data_IgnoreCase:THI_Event;
    _data_Multiline:THI_Event;
    _data_Pos:THI_Event;

    _event_onExecute:THI_Event;
    _event_onReplace:THI_Event;
    _event_onTest:THI_Event;
    _event_onErr:THI_Event;
    _event_onCount:THI_Event;
    _event_onProgress:THI_Event;
    _event_onPosMax:THI_Event;
    _event_onEnd:THI_Event;

    constructor Create;
    destructor Destroy; override;
    procedure Init;
    procedure _work_doExecute(var _Data:TData; Index:word);
    procedure _work_doReplace(var _Data:TData; Index:word);
    procedure _work_doTest(var _Data:TData; Index:word);
    procedure _work_doPattern(var _Data:TData; Index:word);
    procedure _work_doPatternList(var _Data:TData; Index:word);
    procedure _work_doSetIndexPatternLst(var _Data:TData; Index:word);
    procedure _work_doPause(var _Data:TData; Index:word);

    procedure _var_Value(var _Data:TData; Index:word);
    procedure _var_Position(var _Data:TData; Index:word);
    procedure _var_Length(var _Data:TData; Index:word);

end;


implementation

constructor THIRegExp.Create;
begin
  inherited Create;
  OleInit;
  fList := NewStrList;
  InitAdd(Init);
end;

destructor THIRegExp.Destroy;
begin
  objRegExp.Free;
  objMatches.Free;
  objMatch.Free;
  fList.Free;
  inherited Destroy;
end;

procedure THIRegExp.SetProp(var obj: OleVariant);
var dt:TData;
begin
  obj.Global := ReadInteger(dt,_data_Global,_prop_Global);
  obj.IgnoreCase := ReadInteger(dt,_data_IgnoreCase,_prop_IgnoreCase);
  obj.Multiline := ReadInteger(dt,_data_Multiline,_prop_Multiline);
end;

procedure THIRegExp.Init;
begin
  try
    objRegExp := CreateOleObject('VBScript.RegExp');
    SetProp(objRegExp);
  except  _hi_onEvent(_event_onErr, 'Error: create obj RegExp');
  end;
  fList.SetText(_prop_PatternList,False);
  fStop := False;
end;

procedure THIRegExp._work_doExecute;
var s:string; i:integer;
begin
  fPattern := ReadString(_Data,_data_Pattern,_prop_Pattern);
  s := ReadString(_Data,_data_strSource,'');
  if (s = '') or (fPattern = '') then
    begin
      _hi_onEvent(_event_onErr, 'Error: Pattern or Source = ""');
      exit;
    end;
  SetProp(objRegExp);
  objRegExp.Pattern := fPattern;
  if objRegExp.Test(s) then
    begin
      try
        objMatches := objRegExp.Execute(s);
      except  _hi_onEvent(_event_onErr, 'Error: create obj Matches');
      end;
      fpos := ReadInteger(_Data,_data_Pos,0);
      fmax := objMatches.Count-1;
      _hi_onEvent(_event_onPosMax,fmax);
      for i := fpos to fmax do
        begin
            if fStop then break;
            objMatch := objMatches.Item[i];
            fValue := objMatch.Value;
            fPosition := objMatch.FirstIndex;
            fLength := objMatch.Length;
            _hi_onEvent(_event_onExecute,fValue);
            _hi_onEvent(_event_onProgress,i);
        end;
    end else _hi_onEvent(_event_onErr, 'Error: test pattern');
    if fStop then exit;
    _hi_onEvent(_event_onEnd, 'End find');
    _hi_onEvent(_event_onProgress,0);
end;

procedure THIRegExp._work_doReplace;
var s,sr,p,res:string;
begin
  p := ReadString(_Data,_data_Pattern,_prop_Pattern);
  s := ReadString(_Data,_data_strSource,'');
  sr := ReadString(_Data,_data_strReplace,'');
  if (s = '') or (sr = '') or (p = '') then
    begin
      _hi_onEvent(_event_onErr, 'Error: Pattern or Source or Replace = ""');
    end;
  SetProp(objRegExp);
  objRegExp.Pattern := p;
  if objRegExp.Test(s) then
    begin
      res := objRegExp.Replace(s,sr);
      _hi_onEvent(_event_onReplace,res);
    end else _hi_onEvent(_event_onErr, 'Error: test pattern');
    _hi_onEvent(_event_onEnd, 'End replace');
end;

procedure THIRegExp._work_doTest;
var s,p:string;
begin
  p := ReadString(_Data,_data_Pattern,_prop_Pattern);
  s := ReadString(_Data,_data_strSource,'');
  if (s = '') or (p = '') then
    begin
      _hi_onEvent(_event_onErr, 'Error: Pattern or Source = ""');
      exit;
    end;
  SetProp(objRegExp);
  objRegExp.Pattern := p;
  if objRegExp.Test(s) then
    _hi_onEvent(_event_onTest,1)
  else
    _hi_onEvent(_event_onTest,0);
end;

procedure THIRegExp._work_doPattern;
begin
  _prop_Pattern := ToString(_Data);
end;

procedure THIRegExp._work_doPatternList;
var s:string;
begin
  s := ToString(_Data);
  fList.Clear;
  fList.SetText(s,False);
end;

procedure THIRegExp._work_doSetIndexPatternLst;
var idx:integer;
begin
  _prop_Pattern := fList.Items[idx];
end;

procedure THIRegExp._work_doPause;
var p:integer;
begin
  p := ToInteger(_Data);
  case p of
   0: fStop := True;
   1: begin
        fStop := False;
        dtString(_Data,fPattern);
        _work_doExecute(_Data,0);
      end;
  end;
end;

procedure THIRegExp._var_Value;
begin
  dtString(_Data,fValue);
end;

procedure THIRegExp._var_Position;
begin
  dtInteger(_Data,fPosition);
end;

procedure THIRegExp._var_Length;
begin
  dtInteger(_Data,fLength);
end;

end.
